package xyz.veronie.bgg.result;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.LinkOption;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;

import xyz.veronie.bgg.ui.helpers.Resources;

public class Thing {

	private int id;
	private ThingMetaData metaData;
	private ThingUserData userData;
	private ThingDetails details;
	
	// cache
	private Image thumbImage;
	private Path imagePath;
		
	public static final String IdHeader = "ID";
	public static final String ThumbHeader = "Thumbnail";
	public static final String NameHeader = "Name";
	public static final String DetailsHeader = "Details";
	public static final String ExportFlagHeader = "Export";
	
	public Thing(int id, ThingMetaData metaData) {
		this.id = id;
		this.metaData = metaData;
	}
	
	
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public ThingMetaData getMetaData() {
		return metaData;
	}
	public void setMetaData(ThingMetaData metaData) {
		this.metaData = metaData;
	}
	
	public void setUserData(ThingUserData userData) {
		this.userData = userData;
	}
	
	public ThingUserData getUserData() {
		return userData;
	}
	
	public ThingDetails getDetails() {
		return details;
	}
	
	public void setDetails(ThingDetails details) {
		this.details = details;
	}
	
	public Image getThumbnail() {
		if(thumbImage == null) {
			getThumbImage(id);
		}
		return thumbImage;
	}
	
	public String getImagePath() {
		if(imagePath != null)
			return imagePath.toString();
		else 
			return "";
	}
	
	private void getThumbImage(int thingId) {
		Path tmpDir = Resources.INSTANCE.getTmpDir();
		if(tmpDir != null) {
			imagePath = Paths.get(Resources.INSTANCE.getThumbsDir().toString()
					+ File.separator + String.valueOf(thingId) + ".png");
			
			if(Files.exists(imagePath, LinkOption.NOFOLLOW_LINKS)) {
				String filename = imagePath.toString().replace("\\","/");
				thumbImage = Resources.INSTANCE.getResourceManager().createImage(
						ImageDescriptor.createFromFile(null, filename));				
			} else {
				try {
					ImageDescriptor descriptor = ImageDescriptor.createFromURL(
							new URL(metaData.getThumbURL()));
					thumbImage = descriptor.createImage();

					ImageLoader saver = new ImageLoader();
					saver.data = new ImageData[] { thumbImage.getImageData() };
					saver.save(imagePath.toString(), SWT.IMAGE_PNG);					
				} catch (MalformedURLException e) {
					System.out.println("INFO: Malformed URL for ThingId " + thingId);
				}
			}

		}
	}


	/// get field at idx, order of titles
    public String getField(int idx) {
    	
    	String returnStr;

    	switch(idx) {
    	case 0:
    		returnStr = "";
    		break;
    	case 1:
    		returnStr = this.getMetaData().getName();
    		break;
    	case 2:
    		returnStr = String.valueOf(this.id);
    		break;
    	case 3:
    		returnStr = (this.getDetails() != null) ? "yes" : "no";
    		break;
    	case 4:
    		returnStr = this.getMetaData().getName();
    		break;
    	default:
    		throw new ArrayIndexOutOfBoundsException(
    				"idx " + String.valueOf(idx) + " must be in [0,4]");
    	}

    	return returnStr;
    }
    
    
    @Override
    public boolean equals(Object o) {
    	if(o instanceof Thing) {
    		return ((Thing)o).getId() == getId();
    	} else {
    		return false;
    	}
    }
	
    
    /// convert the thing into a comma-separated line for a result.txt file
    /**
     * The format is (without line breaks):
     * id,name,designer,publisher,artist,yearpublished,minplayers,maxplayers,playingtime,minplaytime,maxplaytime,
     * age,usersrated,average,bayesaverage,rank,rank_wg,numcomments,numweights,averageweight,stddev,median,
     * owned,trading,wanting,wishing,userrating,image,category,mechanic,comment,
     * 1player,2player,3player,4player,5player,6player,7player,8player,9player,10player,
     * 11player,12player,13player,14player,15player,16player,17player,18player,19player,20player,
     * description,exp,basegame,reimplement,reimplement_name,reimplemented,reimplemented_name,
     * contains,contains_name,iscontained,iscontained_name,integration,integration_name,
     * numplays,price,userweight,wishpriority,expansions,domain,family,age_poll
     * 
     * @return
     */
    public String toResultTxtLine() {
    	StringBuilder str = new StringBuilder();
    	final String emptyItem = "\"\",";
        // id,name,designer,publisher,artist,yearpublished,minplayers,maxplayers,playingtime,minplaytime,maxplaytime,
        // age,usersrated,average,bayesaverage,rank,rank_wg,numcomments,numweights,averageweight,stddev,median,
    	str.append(Integer.toString(id)).append(",");
    	str.append(metaData.getName()).append(",");
    	if(details != null) {
    		str.append(details.designer).append(",");
    		str.append(details.publisher).append(",");
    		str.append(details.artist).append(",");
    		str.append(details.yearpublished).append(",");
    		str.append(details.minplayers).append(",");
    		str.append(details.maxplayers).append(",");
    		str.append(details.playingtime).append(",");
    		str.append(details.minplaytime).append(",");
    		str.append(details.maxplaytime).append(",");
    		str.append(details.age).append(",");
    		str.append(details.usersrated).append(",");
    		str.append(details.average).append(",");
    		str.append(floatToResult(details.bayesaverage)).append(",");
    		str.append(details.rank).append(",");
    		str.append(details.rank_wg).append(",");
    		str.append(details.numcomments).append(",");
    		str.append(details.numweights).append(",");
    		str.append(floatToResult(details.averageweight)).append(",");
    		str.append(floatToResult(details.stddev)).append(",");
    		str.append(floatToResult(details.median)).append(",");
    	} else {
    		for(int i = 0; i < 20; ++i) {
    			str.append(emptyItem);
    		}
    	}
    	
    	// owned,trading,wanting,wishing,userrating,
    	if(userData != null) {
    		str.append(userData.getOwn()?"1":"0").append(",");
    		str.append(userData.getFortrade()?"1":"0").append(",");
    		str.append(userData.getWant()?"1":"0").append(",");
    		str.append(userData.getWishlist()?"1":"0").append(",");
    		str.append(floatToResult(userData.getRating())).append(",");
    	}
    	
    	// image,category,mechanic,comment,
    	str.append(getImagePath()).append(",");
    	if(details != null) {
    		str.append(details.category).append(",");
    		str.append(details.mechanic).append(",");
    	} else str.append(emptyItem).append(emptyItem);
    	if(userData != null) {
    		str.append(userData.getComment()).append(",");
    	} else str.append(emptyItem);
    	
    	// 1player,2player,3player,4player,5player,6player,7player,8player,9player,10player,
        // 11player,12player,13player,14player,15player,16player,17player,18player,19player,20player,
    	if(details != null) {
	    	for(Recommendation r : details.players) {
	    		str.append(r.name()).append(",");
	    	}
    	} else {
    		for(int i = 0; i < 20; ++i) {
    			str.append("N,");
    		}
    	}
    	// description,exp,basegame,reimplement,reimplement_name,reimplemented,reimplemented_name,
        // contains,contains_name,iscontained,iscontained_name,integration,integration_name,
    	if(details != null) {
    		str.append(details.description).append(",");
    		str.append(details.expansion).append(",");
    		str.append(details.basegameId).append(",");
    		str.append(details.reimplementId).append(",");
    		str.append(details.reimplement_name).append(",");
    		str.append(details.reimplementedById).append(",");
    		str.append(details.reimplementedByName).append(",");
    		str.append(details.containsId).append(",");
    		str.append(details.containsName).append(",");
    		str.append(details.iscontained).append(",");
    		str.append(details.iscontained_name).append(",");
    		str.append(details.integration).append(",");
    		str.append(details.integration_name).append(",");
    	} else {
    		for(int i = 0; i < 13; ++i) {
    			str.append(emptyItem);
    		}
    	}
    	
        // numplays,price,userweight,wishpriority,expansions,domain,family,age_poll
    	if(userData != null) {
    		str.append(userData.getNumPlays()).append(",");
    	} else str.append(emptyItem);
    	if(details != null) {
    		str.append(details.price).append(",");
    	} else str.append(emptyItem);
    	// TODO: find out how to get userweight... 
    	str.append(emptyItem);
    	if(userData != null) {
    		str.append(userData.getWishlistpriority()).append(",");
    	} else str.append(emptyItem);
    	if(details != null) {
    		str.append(details.expansions).append(",");
    		str.append(details.domain).append(",");
    		str.append(details.family).append(",");
    		str.append(details.age_poll).append(",");
    	} else {
    		str.append(emptyItem);
    		str.append(emptyItem);
    		str.append(emptyItem);
    		str.append(emptyItem);
    	}
    	
    	return str.toString();
    }


	private String floatToResult(Float rating) {
		if(rating == null) {
			return "";
		} else {
			return Float.toString(rating.floatValue());
		}
	}
}
